Package: purgeR
Type: Package
Title: Inbreeding-Purging Estimation in Pedigreed Populations
Version: 1.8.3
Date: 2026-06-25
Authors@R: c(
    person("Eugenio", "López-Cortegano", email = "elcortegano@protonmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6914-6305")))
Description: Inbreeding-purging analysis of pedigreed populations, including the computation of the inbreeding coefficient, partial, ancestral and purged inbreeding coefficients, and measures of the opportunity of purging related to the individual reduction of inbreeding load.
    In addition, functions to calculate the effective population size and other parameters relevant to population genetics are included.
    See López-Cortegano E. (2022) <doi:10.1093/bioinformatics/btab599>.
URL: https://gitlab.com/elcortegano/purgeR/
BugReports: https://gitlab.com/elcortegano/purgeR/-/work_items/
Encoding: UTF-8
License: GPL-2
Depends: R (>= 3.5.0)
Imports: doSNOW (>= 1.0.19), foreach (>= 1.5.1), parallel, progress (>=
        1.2.2), Rcpp (>= 1.0.5), RcppProgress (>= 0.4.2), shiny (>=
        1.7.3)
LinkingTo: Rcpp, RcppProgress
Suggests: caret (>= 6.0-86), coda (>= 0.19-4), dplyr (>= 1.0.0), e1071
        (>= 1.7-4), ggplot2 (>= 3.3.1), ggraph (>= 2.1.0), glmnet (>=
        4.0-2), gtable (>= 0.3.0), igraph (>= 1.5.0.1), knitr (>=
        1.28), magrittr (>= 1.5), pander (>= 0.6.3), plotly (>=
        4.10.1), plyr (>= 1.8.6), purrr (>= 0.3.4), rmarkdown (>=
        2.14), scales (>= 1.1.1), stringr (>= 1.4.0), testthat (>=
        2.3.2), tibble (>= 3.0.1), tidyr (>= 1.1.0), tidyselect (>=
        1.1.0)
VignetteBuilder: knitr, rmarkdown
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-25 11:05:11 UTC; elcortegano
Author: Eugenio López-Cortegano [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6914-6305>)
Maintainer: Eugenio López-Cortegano <elcortegano@protonmail.com>
Repository: CRAN
Date/Publication: 2026-06-25 12:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-25 13:43:19 UTC; unix
Archs: purgeR.so.dSYM
