# Generated by roxygen2: do not edit by hand

export(exp_F)
export(exp_Fa)
export(exp_g)
export(ip_F)
export(ip_Fa)
export(ip_Fij)
export(ip_g)
export(ip_op)
export(ped_clean)
export(ped_graph)
export(ped_maternal)
export(ped_rename)
export(ped_sort)
export(pop_Na)
export(pop_Nae)
export(pop_Nancestors)
export(pop_Ne)
export(pop_Nf)
export(pop_Nfe)
export(pop_Ng)
export(pop_hwd)
export(pop_t)
export(w_grandoffspring)
export(w_offspring)
export(w_reproductive_value)
import(RcppProgress)
import(doSNOW)
import(foreach)
import(parallel)
import(progress)
importFrom(Rcpp,evalCpp)
useDynLib(purgeR, .registration = TRUE)
