
library("khroma")

# Set a color-blind safe palette
okabe <- khroma::colour("okabe ito")
okabe_palette <- tibble::tibble(COLOR = okabe(6)) %>% 
  dplyr::mutate(COLOR2 = ifelse(COLOR == "#000000", colorspace::lighten(COLOR, amount = 0.25), colorspace::darken(COLOR, amount = 0.25)))
okp <- as.character(okabe_palette$COLOR)
